@@ -1,10 +1,12 @@
 import contextlib
 import json
+import os
 import pathlib
 import shutil
 import subprocess
 import sys
 import tempfile
+import types
 import unittest.mock as mock
 import uuid
 from typing import Any
@@ -15,11 +17,57 @@ import pop.hub
 import yaml
 from idem.exec.init import ExecReturn
 
+# Get a list of all the built-in types
+PRIMITIVE_TYPES = (i for i in types.__dict__.values() if isinstance(i, type))
+
 
 class IdemRunException(Exception):
     ...
 
 
+def generate_acct_key(*, crypt_plugin: str = "fernet") -> str:
+    """
+    Create a new acct_key using the named plugin and return it as a string
+    """
+    hub = pop.hub.Hub()
+    hub.pop.sub.add(dyne_name="acct")
+
+    return hub.crypto[crypt_plugin].generate_key()
+
+
+def generate_acct_file(
+    data, *, acct_key: str, crypt_plugin: str = "fernet"
+) -> pathlib.Path:
+    """
+    Generate an acct_file based
+    """
+    hub = pop.hub.Hub()
+    hub.pop.sub.add(dyne_name="acct")
+
+    # Make sure the acct data is serializable
+    data = ensure_serializable(data)
+
+    acct_str = hub.crypto[crypt_plugin].encrypt(data=data, key=acct_key)
+
+    with tempfile.NamedTemporaryFile(suffix=".yml.fernet", delete=False) as fh:
+        fh.write(acct_str)
+        path = pathlib.Path(fh.name)
+        return path
+
+
+def ensure_serializable(obj):
+    def _default(x):
+        if hasattr(x, "__dict__"):
+            return x.__dict__
+        else:
+            return str(x)
+
+    # Convert the object to a json string
+    json_str = json.dumps(obj, default=_default)
+    # Convert it back to itself
+    return json.loads(json_str)
+
+
 def run_sls(
     sls: List[str],
     runtime: str = "parallel",
@@ -318,16 +366,48 @@ def run_ex(
 
 
 def idem_cli(
-    subcommand: str, *args, env: Dict[str, str] = None, check: bool = True
+    subcommand: str,
+    *args,
+    env: Dict[str, str] = None,
+    check: bool = True,
+    acct_data: Dict[str, Any],
 ) -> Dict[str, Any]:
     """
     Shell out to run an idem subcommand on the cli and parse the output as json
     """
-    if env is None:
-        env = {}
     runpy = pathlib.Path(__file__).parent / "run.py"
     command = [sys.executable, runpy, subcommand, *args, "--output=json"]
-    ret = subprocess.run(command, capture_output=True, encoding="utf-8", env=env)
+
+    if env is None:
+        env = {}
+
+    acct_file = None
+    if acct_data:
+        acct_key = generate_acct_key()
+        acct_file = generate_acct_file(acct_data["profiles"], acct_key=acct_key)
+
+        env["ACCT_KEY"] = acct_key
+        env["ACCT_FILE"] = acct_file.absolute()
+        # The new default profile is the first one defined in the acct data
+        first_provider = next(iter(acct_data["profiles"].values()))
+        first_profile = next(iter(first_provider.keys()))
+        env["ACCT_PROFILE"] = first_profile
+    elif os.environ.get("ACCT_FILE") and os.environ.get("ACCT_KEY"):
+        # These can be easily overwritten by cli flags
+        # make sure that ACCT information for tests is easily propagated
+        if "ACCT_KEY" not in env:
+            env["ACCT_KEY"] = os.environ.get("ACCT_KEY")
+        if "ACCT_FILE" not in env:
+            env["ACCT_FILE"] = os.environ.get("ACCT_FILE")
+        if "ACCT_PROFILE" not in env:
+            env["ACCT_PROFILE"] = os.environ.get("ACCT_PROFILE", "default")
+
+    try:
+        ret = subprocess.run(command, capture_output=True, encoding="utf-8", env=env)
+    finally:
+        if acct_file:
+            acct_file.unlink()
+
     if check:
         assert ret.returncode == 0, ret.stderr
     return json.loads(ret.stdout)
